// TypeScript definitions for OmniPlan 4.2.7 (207.16) on macOS 11.5.1
// Generated on 2021-08-13 00:09:32 +0000

// To use these definitions, save this file as `OmniPlan.d.ts`
// and create a `tsconfig.json` file with compiler settings which indicate
// an appropriate set of implicitly defined TypeScript libraries:
//
// {
//     "compilerOptions": {
//         "lib": ["es7"]
//     }
// }


// Alert

declare class Alert {
    constructor (title: string, message: string);
    show(callback: Function | null): Promise<number>;
    addOption(string: string);
}

// Application

declare class Application {
    openDocument(from: Document | null, url: URL, completed: Function);
    readonly buildVersion: Version;
    readonly commandKeyDown: boolean;
    readonly controlKeyDown: boolean;
    readonly name: string;
    readonly optionKeyDown: boolean;
    readonly platformName: string;
    readonly shiftKeyDown: boolean;
    readonly userVersion: Version;
    readonly version: string;
}

// Assignment

declare class Assignment {
    remove();
    readonly isLocal: boolean;
    readonly resource: Resource;
    readonly specificAssignments: Array<Assignment>;
    readonly task: Task;
    readonly totalCost: Decimal;
    readonly totalDuration: number;
    readonly totalEffort: number;
    unitsAssigned: number;
}

// Calendar

declare namespace Calendar {
    const buddhist: Calendar;
    const chinese: Calendar;
    const coptic: Calendar;
    const current: Calendar;
    const ethiopicAmeteAlem: Calendar;
    const ethiopicAmeteMihret: Calendar;
    const gregorian: Calendar;
    const hebrew: Calendar;
    const indian: Calendar;
    const islamic: Calendar;
    const islamicCivil: Calendar;
    const islamicTabular: Calendar;
    const islamicUmmAlQura: Calendar;
    const iso8601: Calendar;
    const japanese: Calendar;
    const persian: Calendar;
    const republicOfChina: Calendar;
}

declare class Calendar {
    dateByAddingDateComponents(date: Date, components: DateComponents): Date | null;
    dateFromDateComponents(components: DateComponents): Date | null;
    dateComponentsFromDate(date: Date): DateComponents;
    dateComponentsBetweenDates(start: Date, end: Date): DateComponents;
    startOfDay(date: Date): Date;
    readonly identifier: string;
    readonly locale: Locale | null;
    readonly timeZone: TimeZone;
}

// CalendarEvent

declare class CalendarEvent {
    readonly end: Date;
    readonly start: Date;
    readonly title: string | null;
}

// Color

declare namespace Color {
    function RGB(r: number, g: number, b: number, a: number | null): Color;
    function HSB(h: number, s: number, b: number, a: number | null): Color;
    function White(w: number, a: number | null): Color;
    const black: Color;
    const blue: Color;
    const brown: Color;
    const clear: Color;
    const cyan: Color;
    const darkGray: Color;
    const gray: Color;
    const green: Color;
    const lightGray: Color;
    const magenta: Color;
    const orange: Color;
    const purple: Color;
    const red: Color;
    const white: Color;
    const yellow: Color;
}

declare class Color {
    blend(otherColor: Color, fraction: number): Color | null;
    readonly alpha: number;
    readonly blue: number;
    readonly brightness: number;
    readonly colorSpace: ColorSpace;
    readonly green: number;
    readonly hue: number;
    readonly red: number;
    readonly saturation: number;
    readonly white: number;
}

// ColorSpace

declare namespace ColorSpace {
    const CMYK: ColorSpace;
    const HSB: ColorSpace;
    const Named: ColorSpace;
    const Pattern: ColorSpace;
    const RGB: ColorSpace;
    const White: ColorSpace;
    const all: Array<ColorSpace>;
}

declare class ColorSpace {
}

// Console

declare class Console {
    log(message: Object, additional: Array<Object | null>);
    error(message: Object, additional: Array<Object | null>);
    info(message: Object, additional: Array<Object | null>);
    warn(message: Object, additional: Array<Object | null>);
    clear();
}

// Credentials

declare class Credentials {
    constructor ();
    read(service: string): object | null;
    write(service: string, username: string, password: string);
    remove(service: string);
    readBookmark(service: string): URL.Bookmark | null;
    writeBookmark(service: string, bookmark: URL.Bookmark);
}

// Data

declare namespace Data {
    function fromString(string: string): Data;
    function fromBase64(string: string): Data;
}

declare class Data {
    toString(): string;
    toBase64(): string;
    readonly length: number;
    readonly toObject: Object | null;
}

// DateComponents

declare class DateComponents {
    constructor ();
    readonly date: Date | null;
    day: number | null;
    era: number | null;
    hour: number | null;
    minute: number | null;
    month: number | null;
    nanosecond: number | null;
    second: number | null;
    timeZone: TimeZone | null;
    year: number | null;
}

// Decimal

declare namespace Decimal {
    function fromString(string: string): Decimal;
    const maximum: Decimal;
    const minimum: Decimal;
    const notANumber: Decimal;
    const one: Decimal;
    const zero: Decimal;
}

declare class Decimal {
    toString(): string;
    add(number: Decimal): Decimal;
    subtract(number: Decimal): Decimal;
    multiply(number: Decimal): Decimal;
    divide(number: Decimal): Decimal;
    compare(number: Decimal): number;
    equals(number: Decimal): boolean;
}

// Dependency

declare class Dependency {
    remove();
    readonly dependent: Task;
    dependentAlignedToEnd: boolean;
    kind: DependencyKind;
    leadTimeDuration: Duration | null;
    leadTimePercentage: number | null;
    readonly prerequisite: Task;
    prerequisiteAlignedToEnd: boolean;
}

// DependencyKind

declare namespace DependencyKind {
    const FinishFinish: DependencyKind;
    const FinishStart: DependencyKind;
    const Invalid: DependencyKind;
    const StartFinish: DependencyKind;
    const StartStart: DependencyKind;
    const all: Array<DependencyKind>;
}

declare class DependencyKind {
}

// Device

declare namespace Device {
    const current: Device;
}

declare class Device {
    readonly iOS: boolean;
    readonly iPad: boolean;
    readonly mac: boolean;
    readonly operatingSystemVersion: Version;
    readonly type: DeviceType | null;
}

// DeviceType

declare namespace DeviceType {
    const all: Array<DeviceType>;
    const iPad: DeviceType;
    const iPhone: DeviceType;
    const mac: DeviceType;
}

declare class DeviceType {
}

// Document

declare namespace Document {
    function makeNew(resultFunction: Function | null): Promise<Document>;
    function makeNewAndShow(resultFunction: Function | null): Promise<Document>;
}

declare class Document {
    close(didCancel: Function | null);
    save();
    fileWrapper(type: string | null): FileWrapper;
    makeFileWrapper(baseName: string, type: string | null): Promise<FileWrapper>;
    undo();
    redo();
    show(resultFunction: Function | null);
    readonly canRedo: boolean;
    readonly canUndo: boolean;
    readonly fileType: string | null;
    readonly name: string | null;
    readonly writableTypes: Array<string>;
}

// PlanDocument

declare class PlanDocument extends Document {
    readonly project: Project;
    readonly windows: Array<Window>;
}

// Duration

declare namespace Duration {
    function workSeconds(s: number): Duration;
    function workHours(h: number): Duration;
    function elapsedHourMinSec(h: number, m: number, s: number): Duration;
    function elapsedDays(d: number): Duration;
    function elapsedYearMonthDay(y: number, m: number, d: number): Duration;
}

declare class Duration {
    readonly elapsed: boolean;
    readonly elapsedDays: number;
    readonly elapsedHours: number;
    readonly elapsedMinutes: number;
    readonly elapsedMonths: number;
    readonly elapsedSeconds: number;
    readonly elapsedWeeks: number;
    readonly elapsedYears: number;
    readonly workSeconds: number;
}

// Email

declare class Email {
    constructor ();
    generate();
    blindCarbonCopy: string | Array<string> | null;
    body: string | null;
    carbonCopy: string | Array<string> | null;
    fileWrappers: Array<FileWrapper>;
    receiver: string | Array<string> | null;
    subject: string | null;
}

// FilePicker

declare class FilePicker {
    constructor ();
    show(): Promise<Array<URL>>;
    folders: boolean;
    message: string;
    multiple: boolean;
    types: Array<TypeIdentifier> | null;
}

// FileSaver

declare class FileSaver {
    constructor ();
    show(fileWrapper: FileWrapper): Promise<URL>;
    message: string;
    nameLabel: string;
    prompt: string;
    types: Array<TypeIdentifier> | null;
}

// FileWrapper

declare namespace FileWrapper {
    function withContents(name: string | null, contents: Data): FileWrapper;
    function withChildren(name: string | null, children: Array<FileWrapper>): FileWrapper;
}

declare class FileWrapper {
    filenameForChild(child: FileWrapper): string | null;
    readonly children: Array<FileWrapper>;
    readonly contents: Data | null;
    readonly destination: URL | null;
    filename: string | null;
    preferredFilename: string | null;
    readonly type: FileWrapper.Type;
}

// FileWrapper.Type

declare namespace FileWrapper.Type {
    const Directory: FileWrapper.Type;
    const File: FileWrapper.Type;
    const Link: FileWrapper.Type;
    const all: Array<FileWrapper.Type>;
}

declare namespace FileWrapper {
    class Type {
    }
}

// Form

declare class Form {
    constructor ();
    addField(field: Form.Field, index: number | null);
    removeField(field: Form.Field);
    show(title: string, confirmTitle: string): Promise<Form>;
    readonly fields: Array<Form.Field>;
    validate: Function | null;
    readonly values: Object;
}

// Form.Field

declare namespace Form {
    class Field {
        readonly displayName: string | null;
        readonly key: string;
    }
}

// Form.Field.Checkbox

declare namespace Form.Field {
    class Checkbox extends Form.Field {
        constructor (key: string, displayName: string | null, value: boolean | null);
    }
}

// Form.Field.Date

declare namespace Form.Field {
    class Date extends Form.Field {
        constructor (key: string, displayName: string | null, value: Date | null, formatter: Formatter.Date | null);
    }
}

// Form.Field.MultipleOptions

declare namespace Form.Field {
    class MultipleOptions extends Form.Field {
        constructor (key: string, displayName: string | null, options: Array<Object>, names: Array<string> | null, selected: Array<Object>);
    }
}

// Form.Field.Option

declare namespace Form.Field {
    class Option extends Form.Field {
        constructor (key: string, displayName: string | null, options: Array<Object>, names: Array<string> | null, selected: Object | null, nullOptionTitle: string | null);
        allowsNull: boolean;
        nullOptionTitle: string | null;
    }
}

// Form.Field.Password

declare namespace Form.Field {
    class Password extends Form.Field {
        constructor (key: string, displayName: string | null, value: string | null);
    }
}

// Form.Field.String

declare namespace Form.Field {
    class String extends Form.Field {
        constructor (key: string, displayName: string | null, value: Object | null, formatter: Formatter | null);
    }
}

// Formatter

declare class Formatter {
}

// Formatter.Date

declare namespace Formatter.Date {
    function withStyle(dateStyle: Formatter.Date.Style, timeStyle: Formatter.Date.Style | null): Formatter.Date;
    function withFormat(format: string): Formatter.Date;
    const iso8601: Formatter.Date;
}

declare namespace Formatter {
    class Date extends Formatter {
        stringFromDate(date: Date): string;
        dateFromString(string: string): Date | null;
        calendar: Calendar;
        readonly dateFormat: string;
        locale: Locale;
        timeZone: TimeZone;
    }
}

// Formatter.Decimal

declare namespace Formatter.Decimal {
    function currency(code: string | null): Formatter.Decimal;
    const currencyCodes: Array<string>;
    const custom: Formatter.Decimal;
    const decimal: Formatter.Decimal;
    const percent: Formatter.Decimal;
    const percentWithDecimal: Formatter.Decimal;
    const plain: Formatter.Decimal;
    const thousandsAndDecimal: Formatter.Decimal;
}

declare namespace Formatter {
    class Decimal extends Formatter {
        stringFromDecimal(number: Decimal): string | null;
        decimalFromString(string: string): Decimal | null;
        decimalSeparator: string;
        negativeFormat: string;
        positiveFormat: string;
        thousandsSeparator: string | null;
        zeroSymbol: string | null;
    }
}

// Formatter.Duration

declare namespace Formatter {
    class Duration extends Formatter {
        constructor ();
        stringFromDecimal(number: Decimal): string | null;
        decimalFromString(string: string): Decimal | null;
        hoursPerDay: number;
        hoursPerWeek: number;
        useVerboseFormat: boolean;
    }
}

// Formatter.Date.Style

declare namespace Formatter.Date.Style {
    const Full: Formatter.Date.Style;
    const Long: Formatter.Date.Style;
    const Medium: Formatter.Date.Style;
    const Short: Formatter.Date.Style;
    const all: Array<Formatter.Date.Style>;
}

declare namespace Formatter.Date {
    class Style {
    }
}

// Image

declare class Image {
}

// Locale

declare namespace Locale {
    const identifiers: Array<string>;
}

declare class Locale {
    constructor (identifier: string);
    readonly calendar: Calendar;
    readonly currencyCode: string | null;
    readonly identifier: string;
}

// MenuItem

declare class MenuItem {
    checked: boolean;
    label: string;
}

// Pasteboard

declare namespace Pasteboard {
    function makeUnique(): Pasteboard;
    const general: Pasteboard;
}

declare class Pasteboard {
    availableType(types: Array<TypeIdentifier>): TypeIdentifier | null;
    addItems(items: Array<Pasteboard.Item>);
    clear();
    dataForType(type: TypeIdentifier): Data | null;
    setDataForType(data: Data, type: TypeIdentifier);
    stringForType(type: TypeIdentifier): string | null;
    setStringForType(string: string, type: TypeIdentifier);
    URL: URL | null;
    URLs: Array<URL> | null;
    color: Color | null;
    colors: Array<Color> | null;
    readonly hasColors: boolean;
    readonly hasImages: boolean;
    readonly hasStrings: boolean;
    readonly hasURLs: boolean;
    image: Image | null;
    images: Array<Image> | null;
    items: Array<Pasteboard.Item>;
    string: string | null;
    strings: Array<string> | null;
    readonly types: Array<TypeIdentifier>;
}

// Pasteboard.Item

declare namespace Pasteboard {
    class Item {
        constructor ();
        dataForType(type: TypeIdentifier): Data | null;
        setDataForType(data: Data, type: TypeIdentifier);
        stringForType(type: TypeIdentifier): string | null;
        setStringForType(string: string, type: TypeIdentifier);
        readonly types: Array<TypeIdentifier>;
    }
}

// PlugIn

declare namespace PlugIn {
    function find(identifier: string, minimumVersion: Version | null): PlugIn | null;
    const all: Array<PlugIn>;
}

declare class PlugIn {
    library(identifier: string): PlugIn.Library | null;
    action(identifier: string): PlugIn.Action | null;
    handler(identifier: string): PlugIn.Handler | null;
    resourceNamed(name: string): URL | null;
    imageNamed(name: string): Image | null;
    readonly URL: URL | null;
    readonly actions: Array<PlugIn.Action>;
    readonly author: string;
    readonly description: string;
    readonly displayName: string;
    readonly handlers: Array<PlugIn.Handler>;
    readonly identifier: string;
    readonly libraries: Array<PlugIn.Library>;
    readonly version: Version;
}

// PlugIn.Action

declare namespace PlugIn {
    class Action {
        constructor (perform: Function);
        readonly description: string;
        readonly label: string;
        readonly longLabel: string;
        readonly mediumLabel: string;
        readonly name: string;
        readonly paletteLabel: string;
        readonly perform: Function;
        readonly plugIn: PlugIn;
        readonly shortLabel: string;
        validate: Function | null;
    }
}

// PlugIn.Handler

declare namespace PlugIn {
    class Handler {
        constructor (invoke: Function);
        readonly invoke: Function;
        readonly name: string;
        readonly plugIn: PlugIn;
        willAttach: Function | null;
        willDetach: Function | null;
    }
}

// PlugIn.Handler.Registration

declare namespace PlugIn.Handler {
    class Registration {
        remove();
    }
}

// PlugIn.Library

declare namespace PlugIn {
    class Library {
        constructor (version: Version);
        readonly name: string;
        readonly plugIn: PlugIn;
        readonly version: Version;
    }
}

// Preferences

declare class Preferences {
    constructor (identifier: string | null);
    read(key: string): Object | null;
    readBoolean(key: string): boolean;
    readString(key: string): string | null;
    readNumber(key: string): number;
    readDate(key: string): Date | null;
    readData(key: string): Data | null;
    write(key: string, value: boolean | string | number | Date | Data | null);
    remove(key: string);
    readonly identifier: string;
}

// Project

declare class Project {
    customValue(forKey: string): string | null;
    setCustomValue(forKey: string, to: string);
    baselineNamed(name: string): Scenario | null;
    readonly actual: Scenario;
    readonly baselineNames: Array<string>;
    readonly document: PlanDocument | null;
    title: string;
}

// Resource

declare class Resource {
    customValue(forKey: string): string | null;
    setCustomValue(forKey: string, to: string);
    addMember(): Resource;
    descendents(): Array<Resource>;
    remove();
    readonly assignments: Array<Assignment>;
    readonly completedCost: Decimal;
    costPerHour: Decimal;
    costPerUse: Decimal;
    efficiency: number;
    email: string | null;
    readonly members: Array<Resource>;
    name: string;
    note: string;
    readonly schedule: Schedule;
    readonly totalCost: Decimal;
    readonly totalHours: number;
    type: ResourceType;
    readonly uniqueID: number;
    unitsAvailable: number;
}

// ResourceAssignmentType

declare namespace ResourceAssignmentType {
    const adjustAssignedUnits: ResourceAssignmentType;
    const adjustDuration: ResourceAssignmentType;
    const adjustEffort: ResourceAssignmentType;
    const all: Array<ResourceAssignmentType>;
}

declare class ResourceAssignmentType {
}

// ResourceCalendar

declare class ResourceCalendar {
    readonly events: Array<CalendarEvent>;
    readonly name: string;
    readonly overtime: boolean;
}

// ResourceType

declare namespace ResourceType {
    const all: Array<ResourceType>;
    const equipment: ResourceType;
    const group: ResourceType;
    const material: ResourceType;
    const staff: ResourceType;
}

declare class ResourceType {
}

// Scenario

declare class Scenario {
    taskNamed(name: string): Task | null;
    resourceNamed(name: string): Resource | null;
    readonly completed: number;
    endDate: Date;
    hasFixedEndDate: boolean;
    hasGenericDates: boolean;
    readonly milestones: Array<Task>;
    name: string;
    readonly rootResource: Resource;
    readonly rootTask: Task;
    readonly schedule: Schedule;
    startDate: Date;
    readonly totalCost: Decimal;
}

// Schedule

declare class Schedule {
    endDate(start: Date, duration: Duration): Date;
    startDate(end: Date, duration: Duration): Date;
    durationBetween(start: Date, end: Date): Duration;
    readonly calendars: Array<ResourceCalendar>;
}

// Selection

declare class Selection {
    readonly document: PlanDocument | null;
    readonly project: Project | null;
    readonly resources: Array<Resource>;
    readonly tasks: Array<Task>;
    readonly window: Window | null;
}

// SharePanel

declare class SharePanel {
    constructor (items: Array<URL | string | Image | FileWrapper>);
    addItem(shareItem: URL | string | Image | FileWrapper);
    addItems(shareItems: Array<URL | string | Image | FileWrapper>);
    removeItem(shareItem: URL | string | Image | FileWrapper);
    removeItems(shareItems: Array<URL | string | Image | FileWrapper>);
    clearItems();
    show();
    items: Array<URL | string | Image | FileWrapper>;
}

// Task

declare class Task {
    customValue(forKey: string): string | null;
    setCustomValue(forKey: string, to: string);
    descendents(): Array<Task>;
    clearResourceLeveledDate();
    addPrerequisite(to: Task): Dependency;
    addDependent(to: Task): Dependency;
    addAssignment(to: Resource): Assignment;
    addSubtask(): Task;
    split(at: Date, resumingAt: Date): boolean;
    remove();
    readonly absoluteEndLimit: Date;
    readonly absoluteStartLimit: Date;
    readonly assignments: Array<Assignment>;
    readonly assignmentsCost: Decimal;
    readonly currentEndLimit: Date;
    readonly currentStartLimit: Date;
    readonly dependents: Array<Dependency>;
    duration: Duration | null;
    effort: number;
    effortDone: number;
    effortRemaining: number;
    readonly endDate: Date;
    endNoEarlierThanDate: Date | null;
    endNoLaterThanDate: Date | null;
    expectedEffortEstimate: number;
    readonly freeSlack: number;
    manualEndDate: Date | null;
    manualStartDate: Date | null;
    maxEffortEstimate: number;
    minEffortEstimate: number;
    note: string;
    readonly prerequisites: Array<Dependency>;
    priority: number;
    resourceAssignmentType: ResourceAssignmentType;
    readonly resourceLeveledDate: Date | null;
    readonly resourceLevelingDelay: number;
    readonly startDate: Date;
    startNoEarlierThanDate: Date | null;
    startNoLaterThanDate: Date | null;
    staticCost: Decimal;
    readonly subtasks: Array<Task>;
    title: string;
    readonly totalCost: Decimal;
    readonly totalSlack: number;
    type: TaskType;
    readonly uniqueID: number;
}

// TaskType

declare namespace TaskType {
    const all: Array<TaskType>;
    const group: TaskType;
    const hammock: TaskType;
    const milestone: TaskType;
    const task: TaskType;
}

declare class TaskType {
}

// TimeZone

declare namespace TimeZone {
    const abbreviations: Array<string>;
}

declare class TimeZone {
    constructor (abbreviation: string);
    readonly abbreviation: string | null;
    readonly daylightSavingTime: boolean;
    readonly secondsFromGMT: number;
}

// Timer

declare namespace Timer {
    function once(interval: number, action: Function): Timer;
    function repeating(interval: number, action: Function): Timer;
}

declare class Timer {
    cancel();
    readonly interval: number;
}

// ToolbarItem

declare class ToolbarItem {
    image: Image | null;
    label: string;
    toolTip: string | null;
}

// TypeIdentifier

declare namespace TypeIdentifier {
    const URL: TypeIdentifier;
    const binaryPropertyList: TypeIdentifier;
    const csv: TypeIdentifier;
    const editableTypes: Array<TypeIdentifier>;
    const gif: TypeIdentifier;
    const image: TypeIdentifier;
    const jpeg: TypeIdentifier;
    const json: TypeIdentifier;
    const pdf: TypeIdentifier;
    const plainText: TypeIdentifier;
    const png: TypeIdentifier;
    const propertyList: TypeIdentifier;
    const readableTypes: Array<TypeIdentifier>;
    const rtf: TypeIdentifier;
    const rtfd: TypeIdentifier;
    const tiff: TypeIdentifier;
    const writableTypes: Array<TypeIdentifier>;
    const xmlPropertyList: TypeIdentifier;
}

declare class TypeIdentifier {
    constructor (identifier: string);
    conformsTo(other: TypeIdentifier): boolean;
    readonly displayName: string;
    readonly identifier: string;
    readonly pathExtensions: Array<string>;
}

// URL

declare namespace URL {
    function choose(types: Array<string>): URL | null;
    function chooseFolder(): URL | null;
    function fromString(string: string): URL | null;
    function tellScript(app: string, js: string, arg: Object | null): URL | null;
    function tellFunction(app: string, jsFunction: Function, arg: Object | null): URL | null;
    const currentAppScheme: string;
}

declare class URL {
    fetch(success: Function, failure: Function | null);
    call(success: Function, failure: Function | null);
    open();
    find(types: Array<TypeIdentifier>, recurse: boolean | null): Promise<Array<URL>>;
    toString(): string;
    appendingPathComponent(component: string): URL;
    deletingLastPathComponent(): URL;
    readonly string: string;
    readonly toObject: Object | null;
}

// URL.Access

declare namespace URL {
    class Access {
        readonly url: URL;
    }
}

// URL.Bookmark

declare namespace URL.Bookmark {
    function fromURL(url: URL): URL.Bookmark;
}

declare namespace URL {
    class Bookmark {
        access(): Promise<URL.Access>;
    }
}

// URL.FetchRequest

declare namespace URL.FetchRequest {
    function fromString(string: string): URL.FetchRequest | null;
}

declare namespace URL {
    class FetchRequest {
        constructor ();
        fetch(): Promise<URL.FetchResponse>;
        bodyData: Data | null;
        bodyString: string | null;
        cache: string | null;
        headers: object;
        method: string | null;
        url: URL | null;
    }
}

// URL.FetchResponse

declare namespace URL {
    class FetchResponse {
        readonly bodyData: Data | null;
        readonly bodyString: string | null;
        readonly headers: object;
        readonly mimeType: string | null;
        readonly statusCode: number;
        readonly textEncodingName: string | null;
        readonly url: URL | null;
    }
}

// Version

declare class Version {
    constructor (versionString: string);
    equals(version: Version): boolean;
    atLeast(version: Version): boolean;
    isAfter(version: Version): boolean;
    isBefore(version: Version): boolean;
    readonly versionString: string;
}

// Window

declare class Window {
    readonly selection: Selection;
}